"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = void 0;
exports.handler = handler;
exports.rcOptionsTypes = rcOptionsTypes;
exports.cliOptionsTypes = cliOptionsTypes;
exports.help = help;
const render_help_1 = __importDefault(require("render-help"));
const cli_utils_1 = require("@pnpm/cli-utils");
const config_1 = require("@pnpm/config");
const error_1 = require("@pnpm/error");
const pick_1 = __importDefault(require("ramda/src/pick"));
const dlx = __importStar(require("./dlx"));
exports.commandNames = ['create'];
async function handler(_opts, params) {
    const [packageName, ...packageArgs] = params;
    if (packageName === undefined) {
        throw new error_1.PnpmError('MISSING_ARGS', 'Missing the template package name.\n' +
            'The correct usage is `pnpm create <name>` ' +
            'with <name> substituted for a package name.');
    }
    const createPackageName = convertToCreateName(packageName);
    return dlx.handler(_opts, [createPackageName, ...packageArgs]);
}
function rcOptionsTypes() {
    return {
        ...(0, pick_1.default)([
            'use-node-version',
        ], config_1.types),
    };
}
function cliOptionsTypes() {
    return {
        ...rcOptionsTypes(),
    };
}
function help() {
    return (0, render_help_1.default)({
        description: 'Creates a project from a `create-*` starter kit.',
        url: (0, cli_utils_1.docsUrl)('create'),
        usages: [
            'pnpm create <name>',
            'pnpm create <name-without-create>',
            'pnpm create <@scope>',
        ],
    });
}
const CREATE_PREFIX = 'create-';
/**
 * Defines the npm's algorithm for resolving a package name
 * for create-* packages.
 *
 * Example:
 *   - `foo`            -> `create-foo`
 *   - `@usr/foo`       -> `@usr/create-foo`
 *   - `@usr`           -> `@usr/create`
 *   - `@usr@2.0.0`     -> `@usr/create@2.0.0`
 *   - `@usr/foo@2.0.0` -> `@usr/create-foo@2.0.0`
 *   - `@usr@latest`    -> `@user/create@latest`
 *
 * For more info, see https://docs.npmjs.com/cli/v9/commands/npm-init#description
 */
function convertToCreateName(packageName) {
    if (packageName[0] === '@') {
        const preferredVersionPosition = packageName.indexOf('@', 1);
        let preferredVersion = '';
        if (preferredVersionPosition > -1) {
            preferredVersion = packageName.substring(preferredVersionPosition);
            packageName = packageName.substring(0, preferredVersionPosition);
        }
        const [scope, scopedPackage = ''] = packageName.split('/');
        if (scopedPackage === '') {
            return `${scope}/create${preferredVersion}`;
        }
        else {
            return `${scope}/${ensureCreatePrefixed(scopedPackage)}${preferredVersion}`;
        }
    }
    else {
        return ensureCreatePrefixed(packageName);
    }
}
function ensureCreatePrefixed(packageName) {
    if (packageName.startsWith(CREATE_PREFIX)) {
        return packageName;
    }
    else {
        return `${CREATE_PREFIX}${packageName}`;
    }
}
//# sourceMappingURL=create.js.map